
#ifndef VTKACCELERATORSVTKMFILTERS_EXPORT_H
#define VTKACCELERATORSVTKMFILTERS_EXPORT_H

#ifdef VTKACCELERATORSVTKMFILTERS_STATIC_DEFINE
#  define VTKACCELERATORSVTKMFILTERS_EXPORT
#  define VTKACCELERATORSVTKMFILTERS_NO_EXPORT
#else
#  ifndef VTKACCELERATORSVTKMFILTERS_EXPORT
#    ifdef AcceleratorsVTKmFilters_EXPORTS
        /* We are building this library */
#      define VTKACCELERATORSVTKMFILTERS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKACCELERATORSVTKMFILTERS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKACCELERATORSVTKMFILTERS_NO_EXPORT
#    define VTKACCELERATORSVTKMFILTERS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKACCELERATORSVTKMFILTERS_DEPRECATED
#  define VTKACCELERATORSVTKMFILTERS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKACCELERATORSVTKMFILTERS_DEPRECATED_EXPORT
#  define VTKACCELERATORSVTKMFILTERS_DEPRECATED_EXPORT VTKACCELERATORSVTKMFILTERS_EXPORT VTKACCELERATORSVTKMFILTERS_DEPRECATED
#endif

#ifndef VTKACCELERATORSVTKMFILTERS_DEPRECATED_NO_EXPORT
#  define VTKACCELERATORSVTKMFILTERS_DEPRECATED_NO_EXPORT VTKACCELERATORSVTKMFILTERS_NO_EXPORT VTKACCELERATORSVTKMFILTERS_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKACCELERATORSVTKMFILTERS_NO_DEPRECATED
#    define VTKACCELERATORSVTKMFILTERS_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: vtkAcceleratorsVTKmFiltersModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"

#endif /* VTKACCELERATORSVTKMFILTERS_EXPORT_H */
