// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#ifndef vtk_xdmf3_h
#define vtk_xdmf3_h

/* Use the xdmf3 library configured for VTK.  */
#define VTK_MODULE_USE_EXTERNAL_vtkxdmf3 0

#ifdef VTK_USE_SYSTEM_XDMF3
#else
#define VTKXDMF3_HEADER(x)  <vtkxdmf3/x>
#endif

#endif
